%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%      Multicarrier transmission
% 
%  Generation of the figure "LP-OFDM"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012


% initialization
clear

% number of simulation cycles
Number_of_channel_realizations        = 1000;
Number_of_data_and_noise_cycles       = 10;
Number_of_data_and_noise_realizations = 100;

% system and channel parameters
A =  2;      % Alphabet size for BPSK
L =  2;      % CP length (same as channel order)
M = 16;      % number of subcarriers (same as FFT size)
N = M + L;   % length of each OFDM block (including CP) 

% precoder parameters
number_of_groups = 2; % Number of subcarrier groups, must be a divisor of M
                      % set 1 for LP-OFDM
                      % set an integer (divisor of M) greater than 1 for LP-OFDM with subcarrier grouping
precoder_size = M / number_of_groups; % Precoder size
alpha_vector  = exp(sqrt(-1)*2*pi*((0:(precoder_size-1))/precoder_size + 1/(4*precoder_size))); % roots for Vandermonde precoder
Theta         = 1/sqrt(precoder_size) * repmat(alpha_vector.',1,precoder_size) .^ repmat(0:(precoder_size-1),precoder_size,1);
% Theta is the Vandermonde precoder matrix for each subcarrier group
indexes       = reshape(reshape(1:M,precoder_size,number_of_groups).',M,1); % indexes for subcarrier grouping
Perm_temp     = eye(M); % Permutation matrix for subcarrier grouping (initialization)
Perm          = Perm_temp(:,indexes); % Permutation matrix for subcarrier grouping
Theta_full    = zeros(M,M); % Precoder matrix (pre-allocation)
for index_group = 1:number_of_groups
    Theta_full( (index_group-1)*precoder_size+1:(index_group*precoder_size),(index_group-1)*precoder_size+1:(index_group*precoder_size) ) = Theta;
end
% Theta_full is the overall precoder matrix (for all subcarriers)

% power-delay profile of the multipath channel
pdp = 1/(L+1) * ones(L+1,1);  % for uniform power-delay profile

% d_candidates is the auxiliary matrix for exhaustive ml search
d_candidates       = ones(precoder_size,2^precoder_size); % initialization
symbol_vector_temp = [-1 1];                              % auxiliary symbol vector
for index_subcarrier = 1:precoder_size,
    d_candidates(precoder_size-index_subcarrier+1,1:(2^precoder_size)) = repmat(symbol_vector_temp ,1,2^(precoder_size-index_subcarrier));
    symbol_vector_temp                                                 = reshape(repmat(symbol_vector_temp ,2,1),1,2^(index_subcarrier+1));
end

% power parameters
   EbN0dB_min  =  0;   % minimum simulated value of Eb/No (in dB)
   EbN0dB_step =  2;   % simulation step for Eb/No (in dB)
   EbN0dB_max  = 18;   % maximum simulated value of Eb/No (in dB)
EbN0dB_vector  = EbN0dB_min:EbN0dB_step:EbN0dB_max;   % vector of Eb/No in dB
EbN0lin_vector = 10.^(0.1*EbN0dB_vector);             % vector of Eb/No in linear scale
EsN0dB_vector  = EbN0dB_vector + 10*log10(log2(A));   % vector of Es/No in dB
EsN0lin_vector = log2(A)*EbN0lin_vector;              % vector of Es/No in linear scale

% ber for each equalizer (pre-allocation)
ber_ofdm = zeros(1,length(EbN0dB_vector));    % for per-subcarrier equalization of conventional OFDM 
ber_zf   = zeros(1,length(EbN0dB_vector));    % for zero-forcing equalization of LP-OFDM
ber_mmse = zeros(1,length(EbN0dB_vector));    % for MMSE equalization of LP-OFDM 
ber_dfe  = zeros(1,length(EbN0dB_vector));    % for decision-feedback equalization of LP-OFDM 
ber_ml   = zeros(1,length(EbN0dB_vector));    % for maximum-likelihood equalization of LP-OFDM

old_percentage = 0; % initialization of the percentage of simulation done
str_part1 = 'Percentage of simulation: ';

% start of simulation cycle (index_chann is the index of the channel realization)
for index_chann = 1:Number_of_channel_realizations
    
    % display the percentage of simulation
    new_percentage = floor((index_chann-1)/Number_of_channel_realizations*100);
    if new_percentage > old_percentage
        str_part2 = num2str(floor((index_chann-1)/Number_of_channel_realizations*100));
        clc;
        display([str_part1, str_part2]);
        old_percentage = new_percentage;
    end
    
    % generation of the current channel realization
              ht = sqrt(pdp/2).*(randn(L+1,1)+sqrt(-1)*randn(L+1,1));   % time-domain channel (CIR)
    hf_no_intrlv = fft(ht,M);                                           % frequency-domain channel (frequency response)
              hf = diag(Perm'*diag(hf_no_intrlv)*Perm);                 % frequency-domain channel for subcarrier grouping
          inv_hf =(hf).^(-1);
    
    % computation of the equalizers for the current channel realization
    H    = zeros(precoder_size,precoder_size,number_of_groups); % system matrix including both channel and precoder (pre-allocation)
    zf   = zeros(precoder_size,precoder_size,number_of_groups);                       % zero-forcing equalizer (pre-allocation)
    mmse = zeros(precoder_size,precoder_size,number_of_groups,length(EbN0dB_vector)); % MMSE equalizer (pre-allocation)
    W    = zeros(precoder_size,precoder_size,number_of_groups,length(EbN0dB_vector)); % feedforward filter for decision-feedback equalizer (pre-allocation)
    B    = zeros(precoder_size,precoder_size,number_of_groups,length(EbN0dB_vector)); % feedback filter for decision-feedback equalizer (pre-allocation)
    for index_group = 1:number_of_groups
        H(1:precoder_size,1:precoder_size,index_group)  = diag( hf( (index_group*precoder_size-precoder_size+1):(index_group*precoder_size) ) ) * Theta;
        zf(1:precoder_size,1:precoder_size,index_group) = pinv( H(1:precoder_size,1:precoder_size,index_group) );
        for index_ebno = 1:length(EbN0dB_vector)  % index_ebno is the index of Eb/No
            mmse(1:precoder_size,1:precoder_size,index_group,index_ebno) =...
                H(1:precoder_size,1:precoder_size,index_group)' / (1/EsN0lin_vector(index_ebno)*eye(precoder_size) ...
                + H(1:precoder_size,1:precoder_size,index_group) * H(1:precoder_size,1:precoder_size,index_group)');
            X = eye(precoder_size) + EsN0lin_vector(index_ebno) * H(1:precoder_size,1:precoder_size,index_group)' * H(1:precoder_size,1:precoder_size,index_group);
            R = chol(0.5*(X+X'));
            U = R ./ repmat(diag(R),1,precoder_size);
            W(1:precoder_size,1:precoder_size,index_group,index_ebno) = U * mmse(1:precoder_size,1:precoder_size,index_group,index_ebno);
            B(1:precoder_size,1:precoder_size,index_group,index_ebno) = U - diag(diag(U));
        end
    end
    
    % ber, for each equalizer, for the current cycle of data and noise (pre-allocation)
    ber_cycle_ofdm = zeros(1,length(EbN0dB_vector));
    ber_cycle_zf   = zeros(1,length(EbN0dB_vector));
    ber_cycle_mmse = zeros(1,length(EbN0dB_vector));
    ber_cycle_dfe  = zeros(1,length(EbN0dB_vector));
    ber_cycle_ml   = zeros(1,length(EbN0dB_vector));

    % start of inner simulation cycle (index_run is the index of the data and noise realization)    
    for index_run = 1:Number_of_data_and_noise_cycles
        
        % noise generation (normalized to unit power)
        w_norm = 1/sqrt(2) * (randn(M,Number_of_data_and_noise_realizations)+sqrt(-1)*randn(M,Number_of_data_and_noise_realizations));
        
        % symbol generation
          bit1 = randi([0 1],M,Number_of_data_and_noise_realizations); % generation of the bits
             d = 2*bit1 - 1;                                           % generation of the data symbols

        % OFDM signal generation
        y_ofdm = repmat(hf,1,Number_of_data_and_noise_realizations) .* d;

        % LP-OFDM signal generation
        y_prec = diag(hf) * Theta_full * d;
        
        %%%%%%%%%%
        
        % ber, for each equalizer, for the specific realization, for the current Eb/No (pre-allocation)
        ber_ofdm_run = zeros(1,length(EbN0dB_vector));
        ber_zf_run   = zeros(1,length(EbN0dB_vector));
        ber_mmse_run = zeros(1,length(EbN0dB_vector));
        ber_dfe_run  = zeros(1,length(EbN0dB_vector));
        ber_ml_run   = zeros(1,length(EbN0dB_vector));
        
        % start of equalization cycle (index_ebno is the index of Eb/No)
        for index_ebno = 1:length(EbN0dB_vector)
            
            % OFDM received signal, and equalization and data estimation
                                r_ofdm = y_ofdm + 1/sqrt(EsN0lin_vector(index_ebno))*w_norm;
                       d_soft_estimate = repmat(inv_hf,1,Number_of_data_and_noise_realizations) .* r_ofdm;
                         bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);
            ber_ofdm_run(1,index_ebno) = mean(mean(bit1~=bit1_estimate,2));

            % LP-OFDM received signal 
            r_prec = y_prec + 1/sqrt(EsN0lin_vector(index_ebno))*w_norm;
            
            % zero-forcing equalization and data estimation
            for index_group = 1:number_of_groups
                d_soft_estimate((index_group*precoder_size-precoder_size+1):(index_group*precoder_size),1:Number_of_data_and_noise_realizations)= ...
                    zf(1:precoder_size,1:precoder_size,index_group) * r_prec((index_group*precoder_size-precoder_size+1):(index_group*precoder_size),1:Number_of_data_and_noise_realizations);
            end
                       bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);
            ber_zf_run(1,index_ebno) = mean(mean(bit1~=bit1_estimate,2));
            
            % MMSE equalization and data estimation
            for index_group = 1:number_of_groups
                d_soft_estimate((index_group*precoder_size-precoder_size+1):(index_group*precoder_size),1:Number_of_data_and_noise_realizations)= ...
                    mmse(1:precoder_size,1:precoder_size,index_group,index_ebno) * r_prec((index_group*precoder_size-precoder_size+1):(index_group*precoder_size),1:Number_of_data_and_noise_realizations);
            end
                         bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);
            ber_mmse_run(1,index_ebno) = mean(mean(bit1~=bit1_estimate,2));
            
            % decision-feedback equalization and data estimation
            r_dfe_temp    = reshape(r_prec,precoder_size,number_of_groups*Number_of_data_and_noise_realizations);
            bit1_estimate = zeros(M,Number_of_data_and_noise_realizations); % pre-allocation
            
            for index_group = 1:number_of_groups
                for index_block = 1:Number_of_data_and_noise_realizations
                    
                    d_hard_estimate_temp  = zeros(precoder_size,1); % pre-allocation
                    d_soft_estimate2_temp = zeros(precoder_size,1); % pre-allocation
                    bit1_estimate_temp    = zeros(precoder_size,1); % pre-allocation
                    d_soft_estimate1_temp = W(1:precoder_size,1:precoder_size,index_group,index_ebno) * r_dfe_temp(1:precoder_size,(index_block-1)*number_of_groups+index_group);
                    
                    d_soft_estimate2_temp(precoder_size) = d_soft_estimate1_temp(precoder_size);
                    bit1_estimate_temp(precoder_size)    = round((sign(real(d_soft_estimate2_temp(precoder_size)))+1)/2);
                    d_hard_estimate_temp(precoder_size)  = 2*bit1_estimate_temp(precoder_size)-1;
                    
                    for index_subcarrier = (precoder_size-1):-1:1
                        
                        interference_previously_decoded         = B(1:precoder_size,1:precoder_size,index_group,index_ebno)*d_hard_estimate_temp;
                        d_soft_estimate2_temp(index_subcarrier) = d_soft_estimate1_temp(index_subcarrier) - interference_previously_decoded(index_subcarrier);
                        bit1_estimate_temp(index_subcarrier)    = round((sign(real(d_soft_estimate2_temp(index_subcarrier)))+1)/2);
                        d_hard_estimate_temp(index_subcarrier)  = 2*bit1_estimate_temp(index_subcarrier) - 1;
                        
                    end
                    
                    bit1_estimate(((index_group-1)*precoder_size+1):(index_group*precoder_size),index_block) = bit1_estimate_temp;
                    
                end
            end
            ber_dfe_run(1,index_ebno) = mean(mean(bit1~=bit1_estimate,2));
            
            
            % maximum-likelihood equalization and data estimation
            r_ml_temp         = reshape(r_prec,precoder_size,number_of_groups*Number_of_data_and_noise_realizations);
            d_ml_temp         = reshape(hf,precoder_size,number_of_groups);
            d_hard_estimate   = zeros(M,Number_of_data_and_noise_realizations); % pre-allocation
            likelihood_values = zeros(2^precoder_size,Number_of_data_and_noise_realizations*number_of_groups); 
            
            for index_group = 1:number_of_groups
                
                H_ml = diag(d_ml_temp(1:precoder_size,index_group)) * Theta;
                
                for index_block = 1:Number_of_data_and_noise_realizations
                    
                    likelihood_values(1:2^precoder_size,(index_block-1)*number_of_groups+index_group) = ...
                        real(2 * real(r_ml_temp(1:precoder_size,(index_block-1)*number_of_groups+index_group)' * H_ml * d_candidates) ...
                        - sum(abs(H_ml * d_candidates).^2,1)).';
                    [max_likelihood_values,d_index] = max(likelihood_values(1:2^precoder_size,(index_block-1)*number_of_groups+index_group));
                    d_hard_estimate(((index_group-1)*precoder_size+1):(index_group*precoder_size),index_block) = d_candidates(1:precoder_size,d_index);
                    
                end
                
            end
            
                       bit1_estimate = round((sign(real(d_hard_estimate))+1)/2);
            ber_ml_run(1,index_ebno) = mean(mean(bit1~=bit1_estimate,2));
                 
        end
        
        % ber estimation update using the results of the current data and noise realizations
        ber_cycle_ofdm = (index_run-1)/index_run * ber_cycle_ofdm + 1/index_run * ber_ofdm_run;
        ber_cycle_zf   = (index_run-1)/index_run * ber_cycle_zf   + 1/index_run * ber_zf_run;
        ber_cycle_mmse = (index_run-1)/index_run * ber_cycle_mmse + 1/index_run * ber_mmse_run;
        ber_cycle_dfe  = (index_run-1)/index_run * ber_cycle_dfe  + 1/index_run * ber_dfe_run;
        ber_cycle_ml   = (index_run-1)/index_run * ber_cycle_ml   + 1/index_run * ber_ml_run;
        
    end
    
    % ber estimation update using the results of the current channel realization
    ber_ofdm = (index_chann-1)/index_chann * ber_ofdm + 1/index_chann * ber_cycle_ofdm;
    ber_zf   = (index_chann-1)/index_chann * ber_zf   + 1/index_chann * ber_cycle_zf;
    ber_mmse = (index_chann-1)/index_chann * ber_mmse + 1/index_chann * ber_cycle_mmse;
    ber_dfe  = (index_chann-1)/index_chann * ber_dfe  + 1/index_chann * ber_cycle_dfe;
    ber_ml   = (index_chann-1)/index_chann * ber_ml   + 1/index_chann * ber_cycle_ml;
    
end

clc;
display([str_part1, '100']);

% figure LP-OFDM
figure
semilogy(EbN0dB_vector,ber_ofdm,'c*-')
hold on
semilogy(EbN0dB_vector,ber_zf,'r^-')
semilogy(EbN0dB_vector,ber_mmse,'bs-')
semilogy(EbN0dB_vector,ber_dfe,'kv-')
semilogy(EbN0dB_vector,ber_ml,'go-')
axis([EbN0dB_min EbN0dB_max 1e-4 1])
grid on
xlabel('E_b / N_0  (dB)')
ylabel('BER')
legend('OFDM','LP-OFDM (ZF)','LP-OFDM (MMSE)','LP-OFDM (DFE)','LP-OFDM (ML)')


% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
% 
% [2] Z. Wang, G. B. Giannakis, Complex-field coding for OFDM over fading
%     wireless channels, IEEE Trans. Inf. Theory, vol. 49, no. 3,
%     pp. 707-720, Mar. 2003.
%
% [3] Z. Liu, Y. Xin, G. B. Giannakis, Linear constellation precoding for
%     OFDM with maximum multipath diversity and coding gains, IEEE Trans.
%     Commun., vol. 51, no. 3, pp. 416-427, Mar. 2003.

