%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%           Multicarrier transmission
%
%  Generation of the figure "Performance of
%                            Channel Estimation"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012


% initialization
clear

% number of simulation cycles
Number_of_channel_realizations        = 1000;
Number_of_data_and_noise_cycles       = 10;
Number_of_data_and_noise_realizations = 100;

% system and channel parameters
A =   4;          % Alphabet size for QPSK
L =  15;          % CP length (same as channel order)
M = 256;          % number of subcarriers (same as FFT size)
N = M + L;        % length of each OFDM block (including CP or ZP)

P     = L + 1;           % Number of pilots (should be a divisor of M)
sep   = M/P;             % Pilot separation (in number of subcarriers, must be integer)
loc_p = (1:sep:M).';     % Pilot locations
p_s   = ones(P,1);       % Pilot sequence
D_p   = diag(p_s);       % Pilot matrix
D_inv = diag(p_s.^(-1)); % Pilot-compensation matrix
S_p   = kron(eye(P),[1 zeros(1,sep-1)]); % Pilot selection matrix
index = (1:M).';         % subcarrier indexes
loc_d = index(mod(index,sep)~=1); % data locations

F     = 1/sqrt(M) * fft(eye(M)); % unitary FFT matrix of size M x M
F_L   = F(1:M,1:(L+1));          % normalized FFT matrix of size M x (L+1)
F_p   = F_L(loc_p,1:(L+1));      % normalized FFT matrix of size P x (L+1)
B_p   = sqrt(M) * F_p;           % FFT matrix of size P x (L+1)
% B_inv = pinv(B_p);               % FFT-compensation matrix
B_inv = (B_p' * B_p)^(-1) * (B_p'); % FFT-compensation matrix

% interpolation matrix for piecewise-linear interpolation 
LI_matr = zeros(M,P);
lin1    = (1/sep) * (sep:-1:1).'; 
lin2    = (1/sep) * (0:(sep-1)).';
LI_matr(1:sep,1)       = lin1;
LI_matr((M-sep+1):M,1) = lin2;
for ind_col = 2:P
    LI_matr( (((ind_col-1)*sep+1):(ind_col*sep)),ind_col )     = lin1; 
    LI_matr( (((ind_col-2)*sep+1):((ind_col-1)*sep)),ind_col ) = lin2; 
end

% power-delay profile of the multipath channel
lambda = 0.6; pdp = exp( -lambda*(0:L).' ) / sum( exp( -lambda*(0:L).' ) ); % an exponentially decaying power-delay profile
% pdp = 1/(L+1) * ones(L+1,1);  % for uniform power-delay profile

% power parameters
EbN0dB_min  =  0;   % minimum simulated value of Eb/No (in dB)
EbN0dB_step =  5;   % simulation step for Eb/No (in dB)
EbN0dB_max  = 30;   % maximum simulated value of Eb/No (in dB)
EbN0dB_vector  = EbN0dB_min:EbN0dB_step:EbN0dB_max;   % vector of Eb/No in dB
EbN0lin_vector = 10.^(0.1*EbN0dB_vector);             % vector of Eb/No in linear scale
EsN0dB_vector  = EbN0dB_vector + 10*log10(log2(A));   % vector of Es/No in dB
EsN0lin_vector = log2(A)*EbN0lin_vector;              % vector of Es/No in linear scale

% computation of inverse matrix for MMSE channel estimation
V_inv = zeros(L+1,L+1,length(EbN0dB_vector)); % pre-allocation
for ind_ebno = 1:length(EbN0dB_vector)
    V_inv(1:(L+1),1:(L+1),ind_ebno) = (B_p' * B_p + (1/EsN0lin_vector(ind_ebno)) * diag(pdp.^(-1)) )^(-1);
end

% mse for each estimator (pre-allocation)
mse_mmse = zeros(1,length(EbN0dB_vector));    % for MMSE estimation
mse_ml   = zeros(1,length(EbN0dB_vector));    % for ML estimation
mse_li   = zeros(1,length(EbN0dB_vector));    % for piecewise-linear interpolation

% ber for each estimator (pre-allocation)
ber_perf = zeros(1,length(EbN0dB_vector));    % with perfect channel knowledge
ber_mmse = zeros(1,length(EbN0dB_vector));    % for MMSE estimation
ber_ml   = zeros(1,length(EbN0dB_vector));    % for ML estimation
ber_li   = zeros(1,length(EbN0dB_vector));    % for piecewise-linear interpolation

old_percentage = 0; % initialization of the percentage of simulation done
str_part1 = 'Percentage of simulation: ';

% start of simulation cycle (ind_chann is the index of the channel realization)
for ind_chann = 1:Number_of_channel_realizations
    
    % display the percentage of simulation
    new_percentage = floor((ind_chann-1)/Number_of_channel_realizations*100);
    if new_percentage > old_percentage
        str_part2 = num2str(floor((ind_chann-1)/Number_of_channel_realizations*100));
        clc;
        display([str_part1, str_part2]);
        old_percentage = new_percentage;
    end
    
    % generation of the current channel realization
    ht   = sqrt(pdp/2).*(randn(L+1,1)+sqrt(-1)*randn(L+1,1));   % time-domain channel (CIR)
    hf   = fft(ht,M);                                           % frequency-domain channel (frequency response)
    
    % time-domain circulant channel matrix
    Ht_cp = toeplitz([ht;zeros(M-L-1,1)],[ht(1),zeros(1,M-L-1),fliplr(ht(2:(L+1)).')]);
        
    % transmitted signal (pre-allocation)
    t = zeros(M,Number_of_data_and_noise_realizations);
        
    % mse, for each estimator, for the current cycle of data and noise (pre-allocation)
    mse_cycle_mmse = zeros(1,length(EbN0dB_vector));    % for MMSE estimation
    mse_cycle_ml   = zeros(1,length(EbN0dB_vector));    % for ML estimation
    mse_cycle_li   = zeros(1,length(EbN0dB_vector));    % for piecewise-linear interpolation
    
    % ber, for each estimator, for the current cycle of data and noise (pre-allocation)
    ber_cycle_perf = zeros(1,length(EbN0dB_vector));    % with perfect channel knowledge
    ber_cycle_mmse = zeros(1,length(EbN0dB_vector));    % for MMSE estimation
    ber_cycle_ml   = zeros(1,length(EbN0dB_vector));    % for ML estimation
    ber_cycle_li   = zeros(1,length(EbN0dB_vector));    % for piecewise-linear interpolation
        
    % start of inner simulation cycle (ind_run is the index of the data and noise realization)
    for ind_run = 1:Number_of_data_and_noise_cycles
        
        % noise generation (normalized to unit power)
        w_norm = 1/sqrt(2) * (randn(N,Number_of_data_and_noise_realizations)+sqrt(-1)*randn(N,Number_of_data_and_noise_realizations));
        
        % symbol generation
        bit1 = randi([0 1],M-P,Number_of_data_and_noise_realizations); % generation of the bits on the I component
        bit2 = randi([0 1],M-P,Number_of_data_and_noise_realizations); % generation of the bits on the Q component
           d = 1/sqrt(2) * ((2*bit1-1)+sqrt(-1)*(2*bit2-1));           % generation of the data symbols
        
        t(loc_d,1:Number_of_data_and_noise_realizations) = d;  % place data symbols on the data subcarriers
        t(loc_p,1:Number_of_data_and_noise_realizations) = repmat(p_s,1,Number_of_data_and_noise_realizations);
                                                               % place pilot symbols on the pilot subcarriers
        
        % CP-OFDM signal generation
        y_cp = Ht_cp * (F') * t;
        
        % mse, for each estimator, for the specific realization, for the current Eb/No (pre-allocation)
        mse_mmse_run = zeros(1,length(EbN0dB_vector));    % for MMSE estimation
        mse_ml_run   = zeros(1,length(EbN0dB_vector));    % for ML estimation
        mse_li_run   = zeros(1,length(EbN0dB_vector));    % for piecewise-linear interpolation
        
        % ber, for each estimator, for the specific realization, for the current Eb/No (pre-allocation)
        ber_perf_run = zeros(1,length(EbN0dB_vector));    % with perfect channel knowledge
        ber_mmse_run = zeros(1,length(EbN0dB_vector));    % for MMSE estimation
        ber_ml_run   = zeros(1,length(EbN0dB_vector));    % for ML estimation
        ber_li_run   = zeros(1,length(EbN0dB_vector));    % for piecewise-linear interpolation
        
        % start of estimation and equalization cycle (ind_ebno is the index of Eb/No)
        for ind_ebno = 1:length(EbN0dB_vector)
            
            % CP-OFDM received signal (CP is already removed)
            r_cp = y_cp + sqrt(N./(M*EsN0lin_vector(ind_ebno))) * w_norm((L+1):N,1:Number_of_data_and_noise_realizations);
            
            % FFT
            r_fft = F * r_cp;
            
            % Extract the signal received on the pilot locations
            r_p = S_p * r_fft;
            
            % MMSE channel estimation
                             ht_mmse = V_inv(:,:,ind_ebno) * (B_p') * D_inv * r_p;
                             hf_mmse = fft(ht_mmse,M);
            mse_mmse_run(1,ind_ebno) = mean(mean(abs(repmat(hf,1,Number_of_data_and_noise_realizations)-hf_mmse).^2));
            
            % ML channel estimation
                             ht_ml = B_inv * D_inv * r_p;
                             hf_ml = fft(ht_ml,M);
            mse_ml_run(1,ind_ebno) = mean(mean(abs(repmat(hf,1,Number_of_data_and_noise_realizations)-hf_ml).^2)); 
            
            % Channel estimation by piecewise-linear interpolation
                             hf_li = LI_matr * D_inv * r_p;
            mse_li_run(1,ind_ebno) = mean(mean(abs(repmat(hf,1,Number_of_data_and_noise_realizations)-hf_li).^2)); 
                        
            % per-subcarrier equalization and data estimation with perfect channel knowledge
                          t_estimate = repmat(hf.^(-1),1,Number_of_data_and_noise_realizations) .* r_fft;
                     d_soft_estimate = t_estimate(loc_d,1:Number_of_data_and_noise_realizations);           % symbol estimation
                       bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);                            % bit estimation (I component)
                       bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);                            % bit estimation (Q component)
            ber_perf_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
           
            % per-subcarrier equalization and data estimation with MMSE channel estimate
                          t_estimate = (hf_mmse.^(-1)) .* r_fft;
                     d_soft_estimate = t_estimate(loc_d,1:Number_of_data_and_noise_realizations);           % symbol estimation
                       bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);                            % bit estimation (I component)
                       bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);                            % bit estimation (Q component)
            ber_mmse_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
           
            % per-subcarrier equalization and data estimation with ML channel estimate
                        t_estimate = (hf_ml.^(-1)) .* r_fft;
                   d_soft_estimate = t_estimate(loc_d,1:Number_of_data_and_noise_realizations);           % symbol estimation
                     bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);                            % bit estimation (I component)
                     bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);                            % bit estimation (Q component)
            ber_ml_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
            
            % per-subcarrier equalization and data estimation with piecewise-linear-interpolate channel estimate
                        t_estimate = (hf_li.^(-1)) .* r_fft;
                   d_soft_estimate = t_estimate(loc_d,1:Number_of_data_and_noise_realizations);           % symbol estimation
                     bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);                            % bit estimation (I component)
                     bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);                            % bit estimation (Q component)
            ber_li_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
           
        end

        % mse estimation update using the results of the current data and noise realizations
        mse_cycle_mmse = (ind_run-1)/ind_run * mse_cycle_mmse + 1/ind_run * mse_mmse_run;
        mse_cycle_ml   = (ind_run-1)/ind_run * mse_cycle_ml   + 1/ind_run * mse_ml_run;
        mse_cycle_li   = (ind_run-1)/ind_run * mse_cycle_li   + 1/ind_run * mse_li_run;
        
        % ber estimation update using the results of the current data and noise realizations
        ber_cycle_perf = (ind_run-1)/ind_run * ber_cycle_perf + 1/ind_run * ber_perf_run;
        ber_cycle_mmse = (ind_run-1)/ind_run * ber_cycle_mmse + 1/ind_run * ber_mmse_run;
        ber_cycle_ml   = (ind_run-1)/ind_run * ber_cycle_ml   + 1/ind_run * ber_ml_run;
        ber_cycle_li   = (ind_run-1)/ind_run * ber_cycle_li   + 1/ind_run * ber_li_run;
        
    end
    
    % mse estimation update using the results of the current channel realization
    mse_mmse = (ind_run-1)/ind_run * mse_mmse + 1/ind_run * mse_cycle_mmse;
    mse_ml   = (ind_run-1)/ind_run * mse_ml   + 1/ind_run * mse_cycle_ml;
    mse_li   = (ind_run-1)/ind_run * mse_li   + 1/ind_run * mse_cycle_li;

    % ber estimation update using the results of the current channel realization
    ber_perf = (ind_chann-1)/ind_chann * ber_perf + 1/ind_chann * ber_cycle_perf;
    ber_mmse = (ind_chann-1)/ind_chann * ber_mmse + 1/ind_chann * ber_cycle_mmse;
    ber_ml   = (ind_chann-1)/ind_chann * ber_ml   + 1/ind_chann * ber_cycle_ml;
    ber_li   = (ind_chann-1)/ind_chann * ber_li   + 1/ind_chann * ber_cycle_li;
    
end

% CRLB (achieved by ML estimation)
crlb = ( (L+1)/P ) ./ EsN0lin_vector;

clc;
display([str_part1, '100']);

% figure Perf.ChannEstim (MSE)
figure
semilogy(EsN0dB_vector,mse_mmse,'r^-')
hold on
semilogy(EsN0dB_vector,mse_ml,'bs-')
semilogy(EsN0dB_vector,crlb,'m--')
semilogy(EsN0dB_vector,mse_li,'go-')
axis([min(EsN0dB_vector) max(EsN0dB_vector) 1e-4 1])
grid on
xlabel('SNR  (dB)')
ylabel('MSE')
legend('Perfect CSI', 'MMSE estimation','ML estimation','Piecewise linear interpolation')

% figure Perf.ChannEstim (BER)
figure
semilogy(EbN0dB_vector,ber_perf,'kv-')
hold on
semilogy(EbN0dB_vector,ber_mmse,'r^-')
semilogy(EbN0dB_vector,ber_ml,'bs-')
semilogy(EbN0dB_vector,ber_li,'go-')
axis([EbN0dB_min EbN0dB_max 1e-4 1])
grid on
xlabel('E_b / N_0  (dB)')
ylabel('BER')
legend('Perfect CSI', 'MMSE estimation','ML estimation','Piecewise linear interpolation')


% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
%
% [2] M. Morelli and U. Mengali, A comparison of pilot-aided channel
%     estimation methods for OFDM systems, IEEE Trans. Signal Process.,
%     vol. 49, no. 12, pp. 3065-3073, Dec. 2001.
%

