%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%        Multicarrier transmission
% 
%  Generation of the figures "LTV Equalization"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012


% initialization
clear

% number of simulation cycles
Number_of_channel_realizations        = 2000;
Number_of_data_and_noise_cycles       = 10;
Number_of_data_and_noise_realizations = 100;

% system and channel parameters
    A =   4;       % Alphabet size for QPSK
    L =   7;       % CP length (same as channel order)
    M = 128;       % number of subcarriers (same as FFT size)
    N = M + L;     % length of each OFDM block (including CP or ZP) 
    F = 1/sqrt(M) * fft(eye(M));   % unitary FFT matrix of size M
M_act = 96;        % number of active subcarriers
M_vir = M - M_act; % number of virtual subcarriers

% signal covariance
S_cov = zeros(M,M); % pre-allocation
S_cov((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act)) = eye(M_act);  

% maximum Doppler frequency
normalized_Doppler_frequency = 0.1;     % with respect to the subcarrier spacing
f_d = normalized_Doppler_frequency / M; 

% window generation and related definitions
Q1      = 1;
win1    = mbae_soe_win_gen(M,f_d,Q1);
C_win1  = F*diag(win1)*F';
CA_win1 = C_win1((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act));

Q2      = 2;
win2    = mbae_soe_win_gen(M,f_d,Q2);
C_win2  = F*diag(win2)*F';
CA_win2 = C_win2((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act));

% masks for (circularly) banded matrices
mask1 = toeplitz([ones(Q1+1,1);zeros(M-(2*Q1+1),1);ones(Q1,1)]);
mask2 = toeplitz([ones(Q2+1,1);zeros(M-(2*Q2+1),1);ones(Q2,1)]);

% power-delay profile of the multipath channel
pdp = 1/(L+1) * ones(L+1,1);  % for uniform power-delay profile
% lambda = 0.6; pdp = exp( -lambda*(0:L).' ) / sum( exp( -lambda*(0:L).') ); % for exponential power-delay profile

% power parameters
   EbN0dB_min  =  5;   % minimum simulated value of Eb/No (in dB)
   EbN0dB_step =  5;   % simulation step for Eb/No (in dB)
   EbN0dB_max  = 35;   % maximum simulated value of Eb/No (in dB)
EbN0dB_vector  = EbN0dB_min:EbN0dB_step:EbN0dB_max;   % vector of Eb/No in dB
EbN0lin_vector = 10.^(0.1*EbN0dB_vector);             % vector of Eb/No in linear scale
EsN0dB_vector  = EbN0dB_vector + 10*log10(log2(A));   % vector of Es/No in dB
EsN0lin_vector = log2(A)*EbN0lin_vector;              % vector of Es/No in linear scale

% ber for each equalizer (pre-allocation)
ber_pse     = zeros(1,length(EbN0dB_vector));    % for per-subcarrier equalization 
ber_zf      = zeros(1,length(EbN0dB_vector));    % for ZF linear equalization 
ber_zf_b1   = zeros(1,length(EbN0dB_vector));    % for ZF linear equalization (banded with Q = Q1)
ber_zf_b2   = zeros(1,length(EbN0dB_vector));    % for ZF linear equalization (banded with Q = Q2)
ber_zf_w1   = zeros(1,length(EbN0dB_vector));    % for ZF linear equalization (banded and windowed with Q = Q1)
ber_zf_w2   = zeros(1,length(EbN0dB_vector));    % for ZF linear equalization (banded and windowed with Q = Q2)
ber_mmse    = zeros(1,length(EbN0dB_vector));    % for MMSE linear equalization 
ber_mmse_b1 = zeros(1,length(EbN0dB_vector));    % for MMSE linear equalization (banded with Q = Q1)
ber_mmse_b2 = zeros(1,length(EbN0dB_vector));    % for MMSE linear equalization (banded with Q = Q2)
ber_mmse_w1 = zeros(1,length(EbN0dB_vector));    % for MMSE linear equalization (banded and windowed with Q = Q1)
ber_mmse_w2 = zeros(1,length(EbN0dB_vector));    % for MMSE linear equalization (banded and windowed with Q = Q2)
ber_dfe     = zeros(1,length(EbN0dB_vector));    % for MMSE decision-feedback equalization 
ber_dfe_b1  = zeros(1,length(EbN0dB_vector));    % for MMSE decision-feedback equalization (banded with Q = Q1)
ber_dfe_b2  = zeros(1,length(EbN0dB_vector));    % for MMSE decision-feedback equalization (banded with Q = Q2)
ber_dfe_w1  = zeros(1,length(EbN0dB_vector));    % for MMSE decision-feedback equalization (banded and windowed with Q = Q1)
ber_dfe_w2  = zeros(1,length(EbN0dB_vector));    % for MMSE decision-feedback equalization (banded and windowed with Q = Q2)

old_percentage = 0; % initialization of the percentage of simulation done
str_part1 = 'Percentage of simulation: ';

% start of simulation cycle (ind_chann is the index of the channel realization)
for ind_chann = 1:Number_of_channel_realizations
    
    % display the percentage of simulation
    new_percentage = floor((ind_chann-1)/Number_of_channel_realizations*100);
    if new_percentage > old_percentage
        str_part2 = num2str(floor((ind_chann-1)/Number_of_channel_realizations*100));
        clc;
        display([str_part1, str_part2]);
        old_percentage = new_percentage;
    end
   
    % generation of the current channel realization
    ht = ltv_channel_gen(f_d,M,32,L+1,pdp); % time-varying CIR
    Ht = zeros(M,M); % time-domain channel matrix (pre-allocation)
    for index_subcarrier = 1:M
        index_col = mod(((index_subcarrier-L):index_subcarrier)-1,M)+1;
        Ht(index_subcarrier,index_col) = fliplr(ht(1:(L+1),index_subcarrier).'); % time-domain channel matrix
    end
    
    % frequency-domain channel matrices (with mask to enforce banded matrices)
    Hf       = F * Ht * (F');
    Hf_win1  = F * diag(win1) * Ht * (F');
    Hf1      = Hf      .* mask1;
    Hf1_win1 = Hf_win1 .* mask1;
    Hf_win2  = F * diag(win2) * Ht * (F');
    Hf2      = Hf      .* mask2;
    Hf2_win2 = Hf_win2 .* mask2;
    
    % frequency-domain channel matrices (active subcarriers only)
    Hfa       = Hf((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act));
    Hfa1      = Hf1((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act));
    Hfa1_win1 = Hf1_win1((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act));
    Hfa2      = Hf2((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act));
    Hfa2_win2 = Hf2_win2((M_vir/2+1):(M_vir/2+M_act),(M_vir/2+1):(M_vir/2+M_act));    
    
    % computation of the equalizers for the current channel realization
    pse     = diag(diag(Hfa).^(-1));
    
    zf      = pinv(Hfa);
    zf_b1   = pinv(Hfa1);
    zf_b2   = pinv(Hfa2);
    zf_w1   = pinv(Hfa1_win1);
    zf_w2   = pinv(Hfa2_win2);
    
    mmse    = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    mmse_b1 = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    mmse_b2 = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    mmse_w1 = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    mmse_w2 = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    
    fff     = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fff_b1  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fff_b2  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fff_w1  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fff_w2  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    
    fbf     = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fbf_b1  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fbf_b2  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fbf_w1  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    fbf_w2  = zeros(M_act,M_act,length(EbN0dB_vector));   % pre-allocation
    
    for ind_ebno = 1:length(EbN0dB_vector)        % ind_ebno is the index of Eb/No
        
        [mmse(1:M_act,1:M_act,ind_ebno),    fff(1:M_act,1:M_act,ind_ebno),    fbf(1:M_act,1:M_act,ind_ebno)]    = banded_mmse_gen(Hfa, M_act-1,EsN0dB_vector(ind_ebno),1e-7);
        [mmse_b1(1:M_act,1:M_act,ind_ebno), fff_b1(1:M_act,1:M_act,ind_ebno), fbf_b1(1:M_act,1:M_act,ind_ebno)] = banded_mmse_gen(Hfa1,   Q1  ,EsN0dB_vector(ind_ebno),0.005);
        [mmse_b2(1:M_act,1:M_act,ind_ebno), fff_b2(1:M_act,1:M_act,ind_ebno), fbf_b2(1:M_act,1:M_act,ind_ebno)] = banded_mmse_gen(Hfa2,   Q2  ,EsN0dB_vector(ind_ebno),0.005);
        [mmse_w1(1:M_act,1:M_act,ind_ebno), fff_w1(1:M_act,1:M_act,ind_ebno), fbf_w1(1:M_act,1:M_act,ind_ebno)] = banded_windowed_mmse_gen(Hf_win1,win1,M_vir/2,Q1,EsN0dB_vector(ind_ebno),0.005);
        [mmse_w2(1:M_act,1:M_act,ind_ebno), fff_w2(1:M_act,1:M_act,ind_ebno), fbf_w2(1:M_act,1:M_act,ind_ebno)] = banded_windowed_mmse_gen(Hf_win2,win2,M_vir/2,Q2,EsN0dB_vector(ind_ebno),0.005);
                
    end
    
    % ber, for each equalizer, for the current cycle of data and noise (pre-allocation)
    ber_cycle_pse     = zeros(1,length(EbN0dB_vector));
    ber_cycle_zf      = zeros(1,length(EbN0dB_vector));
    ber_cycle_zf_b1   = zeros(1,length(EbN0dB_vector));
    ber_cycle_zf_b2   = zeros(1,length(EbN0dB_vector));
    ber_cycle_zf_w1   = zeros(1,length(EbN0dB_vector));
    ber_cycle_zf_w2   = zeros(1,length(EbN0dB_vector));
    ber_cycle_mmse    = zeros(1,length(EbN0dB_vector));
    ber_cycle_mmse_b1 = zeros(1,length(EbN0dB_vector));
    ber_cycle_mmse_b2 = zeros(1,length(EbN0dB_vector));
    ber_cycle_mmse_w1 = zeros(1,length(EbN0dB_vector));
    ber_cycle_mmse_w2 = zeros(1,length(EbN0dB_vector));
    ber_cycle_dfe     = zeros(1,length(EbN0dB_vector));
    ber_cycle_dfe_b1  = zeros(1,length(EbN0dB_vector));
    ber_cycle_dfe_b2  = zeros(1,length(EbN0dB_vector));
    ber_cycle_dfe_w1  = zeros(1,length(EbN0dB_vector));
    ber_cycle_dfe_w2  = zeros(1,length(EbN0dB_vector));
    
    
    % start of inner simulation cycle (ind_run is the index of the data and noise realization)
    for ind_run = 1:Number_of_data_and_noise_cycles 
                
        % noise generation (normalized to unit power)
        v_norm      = 1/sqrt(2) * (randn(M,Number_of_data_and_noise_realizations)+sqrt(-1)*randn(M,Number_of_data_and_noise_realizations));
        w_norm      = F * v_norm;
        w_norm_win1 = F * diag(win1) * v_norm;
        w_norm_win2 = F * diag(win2) * v_norm;
        
        % symbol generation
        bit1 = randi([0 1],M_act,Number_of_data_and_noise_realizations); % generation of the bits on the I component
        bit2 = randi([0 1],M_act,Number_of_data_and_noise_realizations); % generation of the bits on the Q component
           d = 1/sqrt(2) * ((2*bit1-1)+sqrt(-1)*(2*bit2-1));             % generation of the data symbols
        
        % OFDM signal generation 
        y      =      Hf * [zeros(M_vir/2,Number_of_data_and_noise_realizations);d;zeros(M_vir/2,Number_of_data_and_noise_realizations)];
        y_win1 = Hf_win1 * [zeros(M_vir/2,Number_of_data_and_noise_realizations);d;zeros(M_vir/2,Number_of_data_and_noise_realizations)];
        y_win2 = Hf_win2 * [zeros(M_vir/2,Number_of_data_and_noise_realizations);d;zeros(M_vir/2,Number_of_data_and_noise_realizations)];
        
        % ber, for each equalizer, for the specific realization, for the current Eb/No (pre-allocation)
        ber_pse_run     = zeros(1,length(EbN0dB_vector));
        ber_zf_run      = zeros(1,length(EbN0dB_vector));
        ber_zf_b1_run   = zeros(1,length(EbN0dB_vector));
        ber_zf_b2_run   = zeros(1,length(EbN0dB_vector));
        ber_zf_w1_run   = zeros(1,length(EbN0dB_vector));
        ber_zf_w2_run   = zeros(1,length(EbN0dB_vector));
        ber_mmse_run    = zeros(1,length(EbN0dB_vector));
        ber_mmse_b1_run = zeros(1,length(EbN0dB_vector));
        ber_mmse_b2_run = zeros(1,length(EbN0dB_vector));
        ber_mmse_w1_run = zeros(1,length(EbN0dB_vector));
        ber_mmse_w2_run = zeros(1,length(EbN0dB_vector));
        ber_dfe_run     = zeros(1,length(EbN0dB_vector));
        ber_dfe_b1_run  = zeros(1,length(EbN0dB_vector));
        ber_dfe_b2_run  = zeros(1,length(EbN0dB_vector));
        ber_dfe_w1_run  = zeros(1,length(EbN0dB_vector));
        ber_dfe_w2_run  = zeros(1,length(EbN0dB_vector));
                
        % start of equalization cycle (ind_ebno is the index of Eb/No)
        for ind_ebno = 1:length(EbN0dB_vector)
            
            % received signal (without windowing and with windowing)
            r       = y      + 1/sqrt(EsN0lin_vector(ind_ebno)) * w_norm;
            r_win1  = y_win1 + 1/sqrt(EsN0lin_vector(ind_ebno)) * w_norm_win1;
            r_win2  = y_win2 + 1/sqrt(EsN0lin_vector(ind_ebno)) * w_norm_win2;
            
            % removal of virtual carriers
            ra      =      r((M_vir/2+1):(M_vir/2+M_act),1:Number_of_data_and_noise_realizations);
            ra_win1 = r_win1((M_vir/2+1):(M_vir/2+M_act),1:Number_of_data_and_noise_realizations);
            ra_win2 = r_win2((M_vir/2+1):(M_vir/2+M_act),1:Number_of_data_and_noise_realizations);
            
            % equalization and data estimation
            ber_pse_run(1,ind_ebno)     = ber_estim_qpsk_ble(ra,pse,bit1,bit2);
            
            ber_zf_run(1,ind_ebno)      = ber_estim_qpsk_ble(ra,     zf,   bit1,bit2);
            ber_zf_b1_run(1,ind_ebno)   = ber_estim_qpsk_ble(ra,     zf_b1,bit1,bit2);
            ber_zf_b2_run(1,ind_ebno)   = ber_estim_qpsk_ble(ra,     zf_b2,bit1,bit2);
            ber_zf_w1_run(1,ind_ebno)   = ber_estim_qpsk_ble(ra_win1,zf_w1,bit1,bit2);
            ber_zf_w2_run(1,ind_ebno)   = ber_estim_qpsk_ble(ra_win2,zf_w2,bit1,bit2);

            ber_mmse_run(1,ind_ebno)    = ber_estim_qpsk_ble(ra,        mmse(:,:,ind_ebno),bit1,bit2);
            ber_mmse_b1_run(1,ind_ebno) = ber_estim_qpsk_ble(ra,     mmse_b1(:,:,ind_ebno),bit1,bit2);
            ber_mmse_b2_run(1,ind_ebno) = ber_estim_qpsk_ble(ra,     mmse_b2(:,:,ind_ebno),bit1,bit2);
            ber_mmse_w1_run(1,ind_ebno) = ber_estim_qpsk_ble(ra_win1,mmse_w1(:,:,ind_ebno),bit1,bit2);
            ber_mmse_w2_run(1,ind_ebno) = ber_estim_qpsk_ble(ra_win2,mmse_w2(:,:,ind_ebno),bit1,bit2);
        
            ber_dfe_run(1,ind_ebno)     = ber_estim_qpsk_bdfe(ra,        fff(:,:,ind_ebno),   fbf(:,:,ind_ebno),bit1,bit2);
            ber_dfe_b1_run(1,ind_ebno)  = ber_estim_qpsk_bdfe(ra,     fff_b1(:,:,ind_ebno),fbf_b1(:,:,ind_ebno),bit1,bit2);
            ber_dfe_b2_run(1,ind_ebno)  = ber_estim_qpsk_bdfe(ra,     fff_b2(:,:,ind_ebno),fbf_b2(:,:,ind_ebno),bit1,bit2);
            ber_dfe_w1_run(1,ind_ebno)  = ber_estim_qpsk_bdfe(ra_win1,fff_w1(:,:,ind_ebno),fbf_w1(:,:,ind_ebno),bit1,bit2);
            ber_dfe_w2_run(1,ind_ebno)  = ber_estim_qpsk_bdfe(ra_win2,fff_w2(:,:,ind_ebno),fbf_w2(:,:,ind_ebno),bit1,bit2);

        end

        % ber estimation update using the results of the current data and noise realizations
        ber_cycle_pse     = (ind_run-1)/ind_run * ber_cycle_pse     + 1/ind_run * ber_pse_run;
        
        ber_cycle_zf      = (ind_run-1)/ind_run * ber_cycle_zf      + 1/ind_run * ber_zf_run;
        ber_cycle_zf_b1   = (ind_run-1)/ind_run * ber_cycle_zf_b1   + 1/ind_run * ber_zf_b1_run;
        ber_cycle_zf_b2   = (ind_run-1)/ind_run * ber_cycle_zf_b2   + 1/ind_run * ber_zf_b2_run;
        ber_cycle_zf_w1   = (ind_run-1)/ind_run * ber_cycle_zf_w1   + 1/ind_run * ber_zf_w1_run;
        ber_cycle_zf_w2   = (ind_run-1)/ind_run * ber_cycle_zf_w2   + 1/ind_run * ber_zf_w2_run;
       
        ber_cycle_mmse    = (ind_run-1)/ind_run * ber_cycle_mmse    + 1/ind_run * ber_mmse_run;
        ber_cycle_mmse_b1 = (ind_run-1)/ind_run * ber_cycle_mmse_b1 + 1/ind_run * ber_mmse_b1_run;
        ber_cycle_mmse_b2 = (ind_run-1)/ind_run * ber_cycle_mmse_b2 + 1/ind_run * ber_mmse_b2_run;
        ber_cycle_mmse_w1 = (ind_run-1)/ind_run * ber_cycle_mmse_w1 + 1/ind_run * ber_mmse_w1_run;
        ber_cycle_mmse_w2 = (ind_run-1)/ind_run * ber_cycle_mmse_w2 + 1/ind_run * ber_mmse_w2_run;

        ber_cycle_dfe     = (ind_run-1)/ind_run * ber_cycle_dfe     + 1/ind_run * ber_dfe_run;
        ber_cycle_dfe_b1  = (ind_run-1)/ind_run * ber_cycle_dfe_b1  + 1/ind_run * ber_dfe_b1_run;
        ber_cycle_dfe_b2  = (ind_run-1)/ind_run * ber_cycle_dfe_b2  + 1/ind_run * ber_dfe_b2_run;
        ber_cycle_dfe_w1  = (ind_run-1)/ind_run * ber_cycle_dfe_w1  + 1/ind_run * ber_dfe_w1_run;
        ber_cycle_dfe_w2  = (ind_run-1)/ind_run * ber_cycle_dfe_w2  + 1/ind_run * ber_dfe_w2_run;

    end

    
    % ber estimation update using the results of the current channel realization
    ber_pse     = (ind_chann-1)/ind_chann * ber_pse     + 1/ind_chann * ber_cycle_pse;
    
    ber_zf      = (ind_chann-1)/ind_chann * ber_zf      + 1/ind_chann * ber_cycle_zf;
    ber_zf_b1   = (ind_chann-1)/ind_chann * ber_zf_b1   + 1/ind_chann * ber_cycle_zf_b1;
    ber_zf_b2   = (ind_chann-1)/ind_chann * ber_zf_b2   + 1/ind_chann * ber_cycle_zf_b2;
    ber_zf_w1   = (ind_chann-1)/ind_chann * ber_zf_w1   + 1/ind_chann * ber_cycle_zf_w1;
    ber_zf_w2   = (ind_chann-1)/ind_chann * ber_zf_w2   + 1/ind_chann * ber_cycle_zf_w2;
    
    ber_mmse    = (ind_chann-1)/ind_chann * ber_mmse    + 1/ind_chann * ber_cycle_mmse;
    ber_mmse_b1 = (ind_chann-1)/ind_chann * ber_mmse_b1 + 1/ind_chann * ber_cycle_mmse_b1;
    ber_mmse_b2 = (ind_chann-1)/ind_chann * ber_mmse_b2 + 1/ind_chann * ber_cycle_mmse_b2;
    ber_mmse_w1 = (ind_chann-1)/ind_chann * ber_mmse_w1 + 1/ind_chann * ber_cycle_mmse_w1;
    ber_mmse_w2 = (ind_chann-1)/ind_chann * ber_mmse_w2 + 1/ind_chann * ber_cycle_mmse_w2;
    
    ber_dfe     = (ind_chann-1)/ind_chann * ber_dfe     + 1/ind_chann * ber_cycle_dfe;
    ber_dfe_b1  = (ind_chann-1)/ind_chann * ber_dfe_b1  + 1/ind_chann * ber_cycle_dfe_b1;
    ber_dfe_b2  = (ind_chann-1)/ind_chann * ber_dfe_b2  + 1/ind_chann * ber_cycle_dfe_b2;
    ber_dfe_w1  = (ind_chann-1)/ind_chann * ber_dfe_w1  + 1/ind_chann * ber_cycle_dfe_w1;
    ber_dfe_w2  = (ind_chann-1)/ind_chann * ber_dfe_w2  + 1/ind_chann * ber_cycle_dfe_w2;
    
end

clc;
display([str_part1, '100']);

% figure ZF_&_MMSE (Q = Q1)
figure
semilogy(EbN0dB_vector,ber_pse,'r^-')
hold on
semilogy(EbN0dB_vector,ber_zf,'bs-')
semilogy(EbN0dB_vector,ber_zf_b1,'m*-')
semilogy(EbN0dB_vector,ber_mmse,'kv-')
semilogy(EbN0dB_vector,ber_mmse_b1,'go-')
semilogy(EbN0dB_vector,ber_mmse_w1,'c<-')
axis([EbN0dB_min EbN0dB_max 1e-4 1])
grid on
xlabel('E_b / N_0  (dB)')
ylabel('BER')
legend('Conventional','ZF (full)','ZF (banded)','MMSE (full)','MMSE (banded)','MMSE (windowing)')

% figure MMSE_&_DFE (Q = Q1)
figure
semilogy(EbN0dB_vector,ber_mmse,'kv-')
hold on
semilogy(EbN0dB_vector,ber_mmse_b1,'go-')
semilogy(EbN0dB_vector,ber_mmse_w1,'c<-')
semilogy(EbN0dB_vector,ber_dfe,'r^-')
semilogy(EbN0dB_vector,ber_dfe_b1,'bs-')
semilogy(EbN0dB_vector,ber_dfe_w1,'m*-')
axis([EbN0dB_min EbN0dB_max 1e-4 1])
grid on
xlabel('E_b / N_0  (dB)')
ylabel('BER')
legend('MMSE (full)','MMSE (banded)','MMSE (windowing)','DFE (full)','DFE (banded)','DFE (windowing)')

% figure ZF_&_MMSE (Q = Q2)
figure
semilogy(EbN0dB_vector,ber_pse,'r^-')
hold on
semilogy(EbN0dB_vector,ber_zf,'bs-')
semilogy(EbN0dB_vector,ber_zf_b2,'m*-')
semilogy(EbN0dB_vector,ber_mmse,'kv-')
semilogy(EbN0dB_vector,ber_mmse_b2,'go-')
semilogy(EbN0dB_vector,ber_mmse_w2,'c<-')
axis([EbN0dB_min EbN0dB_max 1e-4 1])
grid on
xlabel('E_b / N_0  (dB)')
ylabel('BER')
legend('Conventional','ZF (full)','ZF (banded)','MMSE (full)','MMSE (banded)','MMSE (windowing)')

% figure MMSE_&_DFE (Q = Q2)
figure
semilogy(EbN0dB_vector,ber_mmse,'kv-')
hold on
semilogy(EbN0dB_vector,ber_mmse_b2,'go-')
semilogy(EbN0dB_vector,ber_mmse_w2,'c<-')
semilogy(EbN0dB_vector,ber_dfe,'r^-')
semilogy(EbN0dB_vector,ber_dfe_b2,'bs-')
semilogy(EbN0dB_vector,ber_dfe_w2,'m*-')
axis([EbN0dB_min EbN0dB_max 1e-4 1])
grid on
xlabel('E_b / N_0  (dB)')
ylabel('BER')
legend('MMSE (full)','MMSE (banded)','MMSE (windowing)','DFE (full)','DFE (banded)','DFE (windowing)')


% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
% 
% [2] L. Rugini, P. Banelli, and G. Leus, Low-complexity banded equalizers
%     for OFDM systems in Doppler spread channels, EURASIP J. Appl. Signal
%     Processing, vol. 2006, Article ID 67404, pp. 1-13, 2006.

