%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%        Multicarrier transmission
% 
%  Generation of the figure "ZP versus CP"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Prepared and maintained by L. Rugini and P. Banelli
% for E-Reference Signal Processing, published by Elsevier
%
% Copyright 2012


% initialization
clear

% number of simulation cycles
Number_of_channel_realizations        = 1000;
Number_of_data_and_noise_cycles       = 10;
Number_of_data_and_noise_realizations = 100;

% system and channel parameters
  A =  4;      % Alphabet size for QPSK
  L = 16;      % CP length (same as channel order)
  M = 64;      % number of subcarriers (same as FFT size)
  N = M + L;   % length of each OFDM block (including CP or ZP) 
  F = 1/sqrt(M) * fft(eye(M));   % unitary FFT matrix of size M
F_N = 1/sqrt(N) * fft(eye(N));   % unitary FFT matrix of size N

% power-delay profile of the multipath channel
lambda = 0.9624; pdp = exp( -lambda*(0:L).' ) / sum( exp( -lambda*(0:L).' ) ); % for Channel A of HIPERLAN/2
% pdp = 1/(L+1) * ones(L+1,1);  % for uniform power-delay profile

% power parameters
   EbN0dB_min  =  0;   % minimum simulated value of Eb/No (in dB)
   EbN0dB_step =  5;   % simulation step for Eb/No (in dB)
   EbN0dB_max  = 40;   % maximum simulated value of Eb/No (in dB)
EbN0dB_vector  = EbN0dB_min:EbN0dB_step:EbN0dB_max;   % vector of Eb/No in dB
EbN0lin_vector = 10.^(0.1*EbN0dB_vector);             % vector of Eb/No in linear scale
EsN0dB_vector  = EbN0dB_vector + 10*log10(log2(A));   % vector of Es/No in dB
EsN0lin_vector = log2(A)*EbN0lin_vector;              % vector of Es/No in linear scale

% ber for each equalizer (pre-allocation)
ber_cp_mmse = zeros(1,length(EbN0dB_vector));    % for per-subcarrier equalization of CP-OFDM 
ber_zp_ola  = zeros(1,length(EbN0dB_vector));    % for OLA equalization of ZP-OFDM 
ber_zp_zf   = zeros(1,length(EbN0dB_vector));    % for ZF equalization of ZP-OFDM 
ber_zp_mmse = zeros(1,length(EbN0dB_vector));    % for MMSE equalization of ZP-OFDM 
ber_zp_fast = zeros(1,length(EbN0dB_vector));    % for fast-MMSE equalization of ZP-OFDM 

old_percentage = 0; % initialization of the percentage of simulation done
str_part1 = 'Percentage of simulation: ';

% start of simulation cycle (ind_chann is the index of the channel realization)
for ind_chann = 1:Number_of_channel_realizations
    
    % display the percentage of simulation
    new_percentage = floor((ind_chann-1)/Number_of_channel_realizations*100);
    if new_percentage > old_percentage
        str_part2 = num2str(floor((ind_chann-1)/Number_of_channel_realizations*100));
        clc;
        display([str_part1, str_part2]);
        old_percentage = new_percentage;
    end
    
    % generation of the current channel realization
    ht   = sqrt(pdp/2).*(randn(L+1,1)+sqrt(-1)*randn(L+1,1));   % time-domain channel (CIR)
    hf   = fft(ht,M);                                           % frequency-domain channel (frequency response)
    
    % time-domain MxM circulant channel matrix for CP-OFDM
    Ht_cp = toeplitz([ht;zeros(M-L-1,1)],[ht(1),zeros(1,M-L-1),fliplr(ht(2:(L+1)).')]);
    
    % time-domain NxM channel matrix for ZP-OFDM
    Ht_zp = toeplitz([ht;zeros(M-1,1)],[ht(1),zeros(1,M-1)]);
    
    % time-domain NxN channel matrix for ZP-OFDM
    Ht_N_zp = toeplitz([ht(L+1);zeros(M-1,1);ht(1:L)],[fliplr(ht.'),zeros(1,M-1,1)]);
    
    % N-point frequency-domain channel
    hf_N = diag((F_N*Ht_N_zp*F_N'));                                           
    
    % computation of the equalizers for the current channel realization
    cp_mmse = diag(hf.^(-1)) * F;                 % per-subcarrier equalizer for CP-OFDM
    zp_ola  = diag(hf.^(-1)) * [F F(1:M,1:L)];    % OLA equalizer for ZP-OFDM
    zp_zf   = F * pinv(Ht_zp);                    % ZF equalizer for ZP-OFDM
    zp_mmse = zeros(M,N,length(EbN0dB_vector));   % MMSE equalizer for ZP-OFDM (pre-allocation)
    zp_fast = zeros(M,N,length(EbN0dB_vector));   % fast-MMSE equalizer for ZP-OFDM (pre-allocation)
    
    for ind_ebno = 1:length(EbN0dB_vector)        % ind_ebno is the index of Eb/No
        
        % MMSE equalizer for ZP
        zp_mmse(1:M,1:N,ind_ebno) = (F * (Ht_zp)' ) / ( Ht_zp * (Ht_zp)' + 1/EsN0lin_vector(ind_ebno) * eye(N) );
        
        % fast-MMSE equalizer for ZP
        zp_fast(1:M,1:N,ind_ebno) = [zeros(M,L) F] * (F_N)' * diag( conj(hf_N) .* ( abs(hf_N).^2 + (N/(M*EsN0lin_vector(ind_ebno))) * ones(N,1) ).^(-1) ) * F_N;
    
    end
    
    % ber, for each equalizer, for the current cycle of data and noise (pre-allocation)
    ber_cycle_cp_mmse = zeros(1,length(EbN0dB_vector));
    ber_cycle_zp_ola  = zeros(1,length(EbN0dB_vector));
    ber_cycle_zp_zf   = zeros(1,length(EbN0dB_vector));
    ber_cycle_zp_mmse = zeros(1,length(EbN0dB_vector));
    ber_cycle_zp_fast = zeros(1,length(EbN0dB_vector));

    % start of inner simulation cycle (ind_run is the index of the data and noise realization)
    for ind_run = 1:Number_of_data_and_noise_cycles  
        
        % noise generation (normalized to unit power)
        w_norm = 1/sqrt(2) * (randn(N,Number_of_data_and_noise_realizations)+sqrt(-1)*randn(N,Number_of_data_and_noise_realizations));
        
        % symbol generation
        bit1 = randi([0 1],M,Number_of_data_and_noise_realizations); % generation of the bits on the I component
        bit2 = randi([0 1],M,Number_of_data_and_noise_realizations); % generation of the bits on the Q component
           d = 1/sqrt(2) * ((2*bit1-1)+sqrt(-1)*(2*bit2-1));         % generation of the data symbols
        
        % CP-OFDM signal generation 
        y_cp = Ht_cp * (F') * d;
        
        % ZP-OFDM signal generation
        y_zp = Ht_zp * (F') * d;
        
        % ber, for each equalizer, for the specific realization, for the current Eb/No (pre-allocation)
        ber_cp_mmse_run = zeros(1,length(EbN0dB_vector));
        ber_zp_ola_run  = zeros(1,length(EbN0dB_vector));
        ber_zp_zf_run   = zeros(1,length(EbN0dB_vector));
        ber_zp_mmse_run = zeros(1,length(EbN0dB_vector));
        ber_zp_fast_run = zeros(1,length(EbN0dB_vector));
        
        % start of equalization cycle (ind_ebno is the index of Eb/No)
        for ind_ebno = 1:length(EbN0dB_vector)
            
            % CP-OFDM received signal 
            r_cp = y_cp + sqrt(N./(M*EsN0lin_vector(ind_ebno))) * w_norm((L+1):N,1:Number_of_data_and_noise_realizations);

            % ZP-OFDM received signal 
            r_zp = y_zp + 1/sqrt(EsN0lin_vector(ind_ebno)) * w_norm(1:N,1:Number_of_data_and_noise_realizations);
                       
            % per-subcarrier equalization and data estimation for CP-OFDM
                        d_soft_estimate = cp_mmse * r_cp;                            % symbol estimation
                          bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);  % bit estimation (I component)
                          bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);  % bit estimation (Q component)
            ber_cp_mmse_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
            
            % OLA equalization and data estimation for ZP-OFDM
                        d_soft_estimate = zp_ola * r_zp;                             % symbol estimation
                          bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);  % bit estimation (I component)
                          bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);  % bit estimation (Q component)
             ber_zp_ola_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
            
            % ZF equalization and data estimation for ZP-OFDM
                        d_soft_estimate = zp_zf * r_zp;                              % symbol estimation
                          bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);  % bit estimation (I component)
                          bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);  % bit estimation (Q component)
              ber_zp_zf_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
            
            % MMSE equalization and data estimation for ZP-OFDM
                        d_soft_estimate = zp_mmse(1:M,1:N,ind_ebno) * r_zp;          % symbol estimation
                          bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);  % bit estimation (I component)
                          bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);  % bit estimation (Q component)
            ber_zp_mmse_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
            
            % fast-MMSE equalization and data estimation for ZP-OFDM
                        d_soft_estimate = zp_fast(1:M,1:N,ind_ebno) * r_zp;          % symbol estimation
                          bit1_estimate = round((sign(real(d_soft_estimate))+1)/2);  % bit estimation (I component)
                          bit2_estimate = round((sign(imag(d_soft_estimate))+1)/2);  % bit estimation (Q component)
            ber_zp_fast_run(1,ind_ebno) = mean(0.5*(mean(bit1~=bit1_estimate,2)+mean(bit2~=bit2_estimate,2))); % ber estimation
                                    
        end
        
        % ber estimation update using the results of the current data and noise realizations
        ber_cycle_cp_mmse = (ind_run-1)/ind_run * ber_cycle_cp_mmse + 1/ind_run * ber_cp_mmse_run;
        ber_cycle_zp_ola  = (ind_run-1)/ind_run * ber_cycle_zp_ola  + 1/ind_run * ber_zp_ola_run;
        ber_cycle_zp_zf   = (ind_run-1)/ind_run * ber_cycle_zp_zf   + 1/ind_run * ber_zp_zf_run;
        ber_cycle_zp_mmse = (ind_run-1)/ind_run * ber_cycle_zp_mmse + 1/ind_run * ber_zp_mmse_run;
        ber_cycle_zp_fast = (ind_run-1)/ind_run * ber_cycle_zp_fast + 1/ind_run * ber_zp_fast_run;
        
    end
    
    % ber estimation update using the results of the current channel realization
    ber_cp_mmse = (ind_chann-1)/ind_chann * ber_cp_mmse + 1/ind_chann * ber_cycle_cp_mmse;
    ber_zp_ola  = (ind_chann-1)/ind_chann * ber_zp_ola  + 1/ind_chann * ber_cycle_zp_ola;
    ber_zp_zf   = (ind_chann-1)/ind_chann * ber_zp_zf   + 1/ind_chann * ber_cycle_zp_zf;
    ber_zp_mmse = (ind_chann-1)/ind_chann * ber_zp_mmse + 1/ind_chann * ber_cycle_zp_mmse;
    ber_zp_fast = (ind_chann-1)/ind_chann * ber_zp_fast + 1/ind_chann * ber_cycle_zp_fast;
    
end

clc;
display([str_part1, '100']);

% figure ZPvsCP
figure
semilogy(EbN0dB_vector,ber_cp_mmse,'r^-')
hold on
semilogy(EbN0dB_vector,ber_zp_ola,'bs-')
semilogy(EbN0dB_vector,ber_zp_zf,'m*-')
semilogy(EbN0dB_vector,ber_zp_mmse,'kv-')
semilogy(EbN0dB_vector,ber_zp_fast,'go-')
axis([EbN0dB_min EbN0dB_max 1e-5 1])
grid on
xlabel('E_b / N_0  (dB)')
ylabel('BER')
legend('CP-OFDM','ZP-OFDM (OLA)','ZP-OFDM (ZF)','ZP-OFDM (MMSE)','ZP-OFDM (fast-MMSE)')


% References
%
% [1] P. Banelli and L. Rugini, OFDM and Multicarrier Signal Processing,
%     E-Reference Signal Processing, Elsevier, to be published in 2013.
% 
% [2] B. Muquet, Z. Wang, G. B. Giannakis, M. de Courville, P. Duhamel,
%     Cyclic prefixing or zero padding for wireless multicarrier transmissions?,
%     IEEE Trans. Commun., vol. 50, no. 12, pp. 2136-2148, Dec. 2002.

